@@ -1,15 +1,17 @@
 """mdpo package."""
 
 from mdpo.md2po import markdown_to_pofile
+from mdpo.md2po2md import markdown_to_pofile_to_markdown
 from mdpo.mdpo2html import markdown_pofile_to_html
 from mdpo.po2md import pofile_to_markdown
 
 
-__version__ = '0.3.54'
+__version__ = '0.3.74'
 __title__ = 'mdpo'
-__description__ = ('Markdown file translation utilities using PO files')
+__description__ = ('Markdown files translation using PO files.')
 __all__ = (
     'markdown_to_pofile',
     'pofile_to_markdown',
     'markdown_pofile_to_html',
+    'markdown_to_pofile_to_markdown',
 )
