@@ -61,19 +61,26 @@ def to_file_content_if_is_file(value, encoding='utf-8'):
     return value
 
 
-def to_glob_or_content(value):
-    """Check if the value passed is a glob or is string content.
+def to_files_or_content(value):
+    """File path/glob/content disambiguator.
+
+    Check if the value passed is a glob, a set of files in a list or is string
+    content.
 
     Args:
-        value (str): Value to check if is a glob or content.
+        value (str): Value to check.
 
     Returns:
-        list: Two values being the first a boolean that indicates if ``value``
-        is a glob (``True``) or content (``False``) and the second value
-        is the content (parsed as glob is first value is ``True``).
+        tuple: Two values being the first a boolean that indicates if ``value``
+        is a list of files (``True``) or string content (``False``) and the
+        second value is the content, which could be an iterator (if a glob or
+        a list of files is passed or a string).
     """
     try:
         parsed = glob.glob(value)
+    except TypeError:
+        # inferes list
+        return (True, value)
     except re.error:
         # some strings like '[s-m]' will produce
         # 're.error: bad character range ... at position'
@@ -82,3 +89,25 @@ def to_glob_or_content(value):
         # assumes it is content
         return (False, value)
     return (True, parsed)
+
+
+def save_file_checking_file_changed(filepath, content, encoding='utf-8'):
+    """Save a file checking if the content has changed.
+
+    Args:
+        pofile (:py:class:`polib.POFile`): POFile to save.
+        po_filepath (str): Path to the new file to save in.
+
+    Returns:
+        bool: If the PO file content has been changed.
+    """
+    if not os.path.isfile(filepath):
+        with open(filepath, 'w', encoding=encoding) as f:
+            f.write(content)
+        return True
+
+    with open(filepath) as f:
+        prev_content = f.read()
+    with open(filepath, 'w', encoding=encoding) as f:
+        f.write(content)
+    return content != prev_content
