@@ -1,5 +1,6 @@
 """Markdown to PO files extractor according to mdpo specification."""
 
+import glob
 import os
 
 import md4c
@@ -9,8 +10,12 @@ from mdpo.command import (
     normalize_mdpo_command_aliases,
     parse_mdpo_html_command,
 )
-from mdpo.event import raise_skip_event
-from mdpo.io import filter_paths, to_glob_or_content
+from mdpo.event import debug_events, raise_skip_event
+from mdpo.io import (
+    filter_paths,
+    save_file_checking_file_changed,
+    to_files_or_content,
+)
 from mdpo.md import parse_link_references
 from mdpo.md4c import (
     DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
@@ -22,11 +27,11 @@ from mdpo.po import (
     po_escaped_string,
     remove_not_found_entries,
 )
-from mdpo.text import min_not_max_chars_in_a_row
+from mdpo.text import min_not_max_chars_in_a_row, parse_wrapwidth_argument
 
 
 class Md2Po:
-    __slots__ = (
+    __slots__ = {
         'filepaths',
         'content',
         'pofile',
@@ -34,7 +39,6 @@ class Md2Po:
         'msgstr',
         'found_entries',
         'disabled_entries',
-        'wrapwidth',
         'ignore_msgids',
         'command_aliases',
         'mark_not_found_as_obsolete',
@@ -59,6 +63,7 @@ class Md2Po:
         '_enable_next_line',
         '_include_next_codeblock',
         '_disable_next_codeblock',
+        '_saved_files_changed',
 
         '_enterspan_replacer',
         '_leavespan_replacer',
@@ -92,6 +97,7 @@ class Md2Po:
         '_inside_htmlblock',
         '_inside_codeblock',
         '_inside_pblock',
+        '_inside_aspan',
         '_inside_liblock',
         '_inside_codespan',
         '_inside_olblock',
@@ -99,22 +105,26 @@ class Md2Po:
         '_quoteblocks_deep',
         '_codespan_start_index',
         '_codespan_backticks',
-        '_current_aspan_target',
+        '_current_aspan_text',
+        '_current_aspan_ref_target',
         '_current_wikilink_target',
         '_current_imgspan',
         '_uls_deep',
         '_link_references',
-    )
+    }
 
-    def __init__(self, glob_or_content, **kwargs):
-        is_glob, glob_or_content = to_glob_or_content(glob_or_content)
+    def __init__(self, files_or_content, **kwargs):
+        is_glob, files_or_content = to_files_or_content(files_or_content)
         if is_glob:
+            filepaths = []
+            for globpath in files_or_content:
+                filepaths.extend(glob.glob(globpath))
             self.filepaths = filter_paths(
-                glob_or_content,
+                filepaths,
                 ignore_paths=kwargs.get('ignore', []),
             )
         else:
-            self.content = glob_or_content
+            self.content = files_or_content
 
         self.pofile = None
         self.po_filepath = None
@@ -125,7 +135,6 @@ class Md2Po:
         self._current_tcomment = None
         self._current_msgctxt = None
 
-        self.wrapwidth = kwargs.get('wrapwidth', 78)
         self.ignore_msgids = kwargs.get('ignore_msgids', [])
         self.command_aliases = normalize_mdpo_command_aliases(
             kwargs.get('command_aliases', {}),
@@ -137,6 +146,8 @@ class Md2Po:
         self.preserve_not_found = kwargs.get('preserve_not_found', True)
 
         self.location = kwargs.get('location', True)
+        # "top level" here because blocks inside blocks are not taken into
+        # account for locations
         self._current_top_level_block_number = 0
         self._current_top_level_block_type = None
         self._current_markdown_filepath = None
@@ -151,6 +162,11 @@ class Md2Po:
                 self.events[event_name] = (
                     [functions] if callable(functions) else functions
                 )
+        if kwargs.get('debug'):
+            for event_name, function in debug_events('md2po').items():
+                if event_name not in self.events:
+                    self.events[event_name] = []
+                self.events[event_name].append(function)
 
         self.plaintext = kwargs.get('plaintext', False)
 
@@ -163,6 +179,10 @@ class Md2Po:
         self._include_next_codeblock = False
         self._disable_next_codeblock = False
 
+        self._saved_files_changed = (
+            False if kwargs.get('_check_saved_files_changed') else None
+        )
+
         self.metadata = {}
 
         if not self.plaintext:
@@ -198,9 +218,6 @@ class Md2Po:
                 self.code_end_string,
             )
 
-            self.link_start_string = kwargs.get('link_start_string', '[')
-            self.link_end_string = kwargs.get('link_end_string', ']')
-
             _include_xheaders = kwargs.get('xheaders', False)
 
             if _include_xheaders:
@@ -211,22 +228,18 @@ class Md2Po:
                     'x-mdpo-italic-end': self.italic_end_string,
                     'x-mdpo-code-start': self.code_start_string,
                     'x-mdpo-code-end': self.code_end_string,
-                    'x-mdpo-link-start': self.link_start_string,
-                    'x-mdpo-link-end': self.link_end_string,
                 })
 
             self._enterspan_replacer = {
                 md4c.SpanType.STRONG.value: self.bold_start_string,
                 md4c.SpanType.EM.value: self.italic_start_string,
                 md4c.SpanType.CODE.value: self.code_start_string,
-                md4c.SpanType.A.value: self.link_start_string,
             }
 
             self._leavespan_replacer = {
                 md4c.SpanType.STRONG.value: self.bold_end_string,
                 md4c.SpanType.EM.value: self.italic_end_string,
                 md4c.SpanType.CODE.value: self.code_end_string,
-                md4c.SpanType.A.value: self.link_end_string,
             }
 
             if 'strikethrough' in self.extensions:
@@ -337,14 +350,20 @@ class Md2Po:
         self._inside_liblock = False
         self._inside_hblock = False
         self._inside_olblock = False
+        self._inside_codespan = False
+
         self._quoteblocks_deep = 0
         self._uls_deep = 0
 
-        self._inside_codespan = False
         self._codespan_start_index = None
         self._codespan_backticks = None
 
-        self._current_aspan_target = None
+        self._inside_aspan = False
+        self._current_aspan_text = ''
+        # indicates the target of the current link, which is referenced and
+        # extracted without using MD4C, so we can preserve it as referenced
+        self._current_aspan_ref_target = None
+
         self._link_references = None
         self._current_wikilink_target = None
         self._current_imgspan = {}
@@ -384,7 +403,7 @@ class Md2Po:
             #       if the user has configured an `enter_block` or
             #       `leave_block` event remembering that
             #       `_current_top_level_block_number` and
-            #       _current_top_level_block_type` properties must be handled
+            #       `_current_top_level_block_type` properties must be handled
             #       accordingly
 
             occurrence = (
@@ -521,8 +540,6 @@ class Md2Po:
         self.command(command, comment, original_command)
 
     def enter_block(self, block, details):
-        # print("ENTER BLOCK:", block.name)
-
         # raise 'enter_block' event
         if raise_skip_event(self.events, 'enter_block', self, block, details):
             return
@@ -606,8 +623,6 @@ class Md2Po:
                 self._current_top_level_block_type = md4c.BlockType.TABLE.value
 
     def leave_block(self, block, details):
-        # print("LEAVE BLOCK:", block.name)
-
         # raise 'leave_block' event
         if raise_skip_event(self.events, 'leave_block', self, block, details):
             return
@@ -637,8 +652,6 @@ class Md2Po:
             self._save_current_msgid()
 
     def enter_span(self, span, details):
-        # print("ENTER SPAN:", span.name, details)
-
         # raise 'enter_span' event
         if raise_skip_event(self.events, 'enter_span', self, span, details):
             return
@@ -646,17 +659,31 @@ class Md2Po:
         if not self.plaintext:
             # underline spans for double '_' character enters two times
             if not self._inside_uspan:
-                try:
-                    self._current_msgid += self._enterspan_replacer[span.value]
-                except KeyError:
-                    pass
+                if self._inside_aspan:  # span inside link text
+                    try:
+                        self._current_aspan_text += self._enterspan_replacer[
+                            span.value
+                        ]
+                    except KeyError:
+                        pass
+                else:
+                    try:
+                        self._current_msgid += (
+                            self._enterspan_replacer[span.value]
+                        )
+                    except KeyError:
+                        pass
 
             if span is md4c.SpanType.A:
+                # here resides the logic of discover if the current link
+                # is referenced
                 if self._link_references is None:
                     self._link_references = parse_link_references(self.content)
 
+                self._inside_aspan = True
+
                 current_aspan_href = details['href'][0][1]
-                self._current_aspan_target = None
+                self._current_aspan_ref_target = None
 
                 if details['title']:
                     current_aspan_title = details['title'][0][1]
@@ -665,12 +692,12 @@ class Md2Po:
                             href == current_aspan_href and
                             title == current_aspan_title
                         ):
-                            self._current_aspan_target = target
+                            self._current_aspan_ref_target = target
                             break
                 else:
                     for target, href, title in self._link_references:
                         if href == current_aspan_href:
-                            self._current_aspan_target = target
+                            self._current_aspan_ref_target = target
                             break
 
             elif span is md4c.SpanType.CODE:
@@ -682,6 +709,9 @@ class Md2Po:
                 # save the index char of the opening backtick
                 self._codespan_start_index = len(self._current_msgid) - 1
             elif span is md4c.SpanType.IMG:
+                if self._link_references is None:
+                    self._link_references = parse_link_references(self.content)
+
                 self._current_imgspan['src'] = details['src'][0][1]
                 self._current_imgspan['title'] = '' if not details['title'] \
                     else details['title'][0][1]
@@ -696,8 +726,6 @@ class Md2Po:
                 self._save_msgid(details['title'][0][1])
 
     def leave_span(self, span, details):
-        # print("LEAVE SPAN:", span.name, details)
-
         # raise 'leave_span' event
         if raise_skip_event(self.events, 'leave_span', self, span, details):
             return
@@ -707,55 +735,102 @@ class Md2Po:
                 if span is md4c.SpanType.WIKILINK:
                     self._current_msgid += self._current_wikilink_target
                     self._current_wikilink_target = None
-
-                try:
-                    self._current_msgid += self._leavespan_replacer[span.value]
-                except KeyError:
-                    pass
+                if self._inside_aspan:  # span inside link text
+                    try:
+                        self._current_aspan_text += self._leavespan_replacer[
+                            span.value
+                        ]
+                    except KeyError:
+                        pass
+                else:
+                    try:
+                        self._current_msgid += (
+                            self._leavespan_replacer[span.value]
+                        )
+                    except KeyError:
+                        pass
 
             if span is md4c.SpanType.A:
-                if self._current_aspan_target:  # reference link
-                    self._current_msgid += f'[{self._current_aspan_target}]'
-                    self._current_aspan_target = None
-                else:
-                    self._current_msgid += '({}{})'.format(
-                        details['href'][0][1],
-                        '' if not details['title'] else ' "{}"'.format(
-                            details['title'][0][1],
-                        ),
+                if self._current_aspan_ref_target:  # referenced link
+                    self._current_msgid += (
+                        f'[{self._current_aspan_text}]'
+                        f'[{self._current_aspan_ref_target}]'
                     )
+                    self._current_aspan_ref_target = None
+                else:
+                    title = details['title'][0][1] if details['title'] else ''
+                    if self._current_aspan_text == details['href'][0][1]:
+                        # autolink vs link clash (see implementation notes)
+                        self._current_msgid += f'<{self._current_aspan_text}'
+                        if title:
+                            self._current_msgid += f' "{title}"'
+                        self._current_msgid += '>'
+                    else:
+                        title_part = f' "{title}"' if title else ''
+                        href = details['href'][0][1]
+                        self._current_msgid += (
+                            f'[{self._current_aspan_text}]({href}{title_part})'
+                        )
+                self._inside_aspan = False
+                self._current_aspan_text = ''
             elif span is md4c.SpanType.CODE:
                 self._inside_codespan = False
                 self._codespan_start_index = None
+
                 # add backticks at the end for escape internal backticks
-                self._current_msgid += (
-                    self._codespan_backticks * self.code_end_string
-                )
+                if self._inside_aspan:
+                    self._current_aspan_text += (
+                        self._codespan_backticks * self.code_end_string
+                    )
+                else:
+                    self._current_msgid += (
+                        self._codespan_backticks * self.code_end_string
+                    )
                 self._codespan_backticks = None
             elif span is md4c.SpanType.IMG:
-                self._current_msgid += '![{}]({}'.format(
-                    self._current_imgspan['text'],
-                    self._current_imgspan['src'],
-                )
-                if self._current_imgspan['title']:
-                    self._current_msgid += ' "%s"' % (
-                        self._current_imgspan['title']
-                    )
-                self._current_msgid += ')'
+                referenced_target, imgspan_title = (None, None)
+                imgspan_src = details['src'][0][1]
+                if details['title']:
+                    imgspan_title = details['title'][0][1]
+                    for target, href, title in self._link_references:
+                        if href == imgspan_src and title == imgspan_title:
+                            referenced_target = target
+                            break
+                else:
+                    for target, href, _ in self._link_references:
+                        if href == imgspan_src:
+                            referenced_target = target
+                            break
+
+                alt_text = self._current_imgspan['text']
+                img_markup = f'![{alt_text}]'
+                if referenced_target:
+                    img_markup += f'[{referenced_target}]'
+                else:
+                    img_markup += f'({imgspan_src}'
+                    if imgspan_title:
+                        img_markup += f' "{imgspan_title}"'
+                    img_markup += ')'
+
                 self._current_imgspan = {}
+
+                if self._inside_aspan:
+                    self._current_aspan_text += img_markup
+                else:
+                    self._current_msgid += img_markup
             elif span is md4c.SpanType.U:
                 self._inside_uspan = False
 
     def text(self, block, text):
-        # print(f"TEXT: '{text}'")
-
         # raise 'text' event
         if raise_skip_event(self.events, 'text', self, block, text):
             return
 
         if not self._inside_htmlblock:
             if not self._inside_codeblock:
-                if self._inside_liblock and text == '\n':
+                if any([  # softbreaks
+                    self._inside_liblock, self._inside_aspan,
+                ]) and text == '\n':
                     text = ' '
                 if not self.plaintext:
                     if self._current_imgspan:
@@ -773,6 +848,12 @@ class Md2Po:
                             self._codespan_backticks * self.code_start_string,
                             self._current_msgid[self._codespan_start_index:],
                         )
+                        if self._inside_aspan:
+                            self._current_aspan_text += text
+                            return
+                    elif self._inside_aspan:
+                        self._current_aspan_text += text
+                        return
                     elif text == self.italic_start_string:
                         text = self.italic_start_string_escaped
                     elif text == self.code_start_string:
@@ -786,9 +867,8 @@ class Md2Po:
                 if self._current_wikilink_target:
                     if text != self._current_wikilink_target:
                         # not self-referenced wikilink
-                        self._current_wikilink_target = '{}|{}'.format(
-                            self._current_wikilink_target,
-                            text,
+                        self._current_wikilink_target = (
+                            f'{self._current_wikilink_target}|{text}'
                         )
                     return
                 self._current_msgid += text
@@ -833,9 +913,21 @@ class Md2Po:
         mo_filepath=None,
         po_encoding=None,
         md_encoding='utf-8',
+        wrapwidth=78,
     ):
         if not po_filepath:
             self.po_filepath = ''
+
+            if save:
+                if os.environ.get('_MDPO_RUNNING') == 'true':
+                    save_arg = '-s/--save'
+                    po_filepath_arg = '-po/--po-filepath'
+                else:
+                    save_arg, po_filepath_arg = ('save', 'po_filepath')
+                raise ValueError(
+                    f"The argument '{save_arg}' does not make sense"
+                    f" without passing the argument '{po_filepath_arg}'.",
+                )
         else:
             self.po_filepath = po_filepath
             if not os.path.exists(po_filepath):
@@ -847,7 +939,7 @@ class Md2Po:
         )
         self.pofile = polib.pofile(
             self.po_filepath,
-            wrapwidth=self.wrapwidth,
+            wrapwidth=parse_wrapwidth_argument(wrapwidth),
             **pofile_kwargs,
         )
 
@@ -876,10 +968,15 @@ class Md2Po:
                 self._current_markdown_filepath = filepath
                 _parse(self.content)
 
+                # reset state
                 self._disable_next_line = False
                 self._disable = False
                 self._enable_next_line = False
+                self._include_next_codeblock = False
+                self._disable_next_codeblock = False
                 self._link_references = None
+                self._current_top_level_block_number = 0
+                self._current_top_level_block_type = None
 
         if self.mark_not_found_as_obsolete:
             mark_not_found_entries_as_obsoletes(
@@ -895,15 +992,22 @@ class Md2Po:
         if self.metadata:
             self.pofile.metadata.update(self.metadata)
 
-        if save and self.po_filepath:
-            self.pofile.save(fpath=self.po_filepath)
+        if save and po_filepath:
+            if self._saved_files_changed is False:
+                self._saved_files_changed = save_file_checking_file_changed(
+                    po_filepath,
+                    str(self.pofile),
+                    encoding=self.pofile.encoding,
+                )
+            else:
+                self.pofile.save(fpath=po_filepath)
         if mo_filepath:
             self.pofile.save_as_mofile(mo_filepath)
         return self.pofile
 
 
 def markdown_to_pofile(
-    glob_or_content,
+    files_or_content,
     ignore=[],
     msgstr='',
     po_filepath=None,
@@ -923,14 +1027,15 @@ def markdown_to_pofile(
     command_aliases={},
     metadata={},
     events={},
+    debug=False,
     **kwargs,
 ):
     """Extracts all the msgids from a string of Markdown content or a group of
     files.
 
     Args:
-        glob_or_content (str): Glob path to Markdown files or a string
-            with valid Markdown content.
+        files_or_content (str, list): Glob path to Markdown files, a list of
+            files or a string with Markdown content.
         ignore (list): Paths of files to ignore. Useful when a glob does not
             fit your requirements indicating the files to extract content.
             Also, filename or a dirname can be defined without indicate the
@@ -942,7 +1047,8 @@ def markdown_to_pofile(
             obsolete if is the case (see ``save`` and
             ``mark_not_found_as_obsolete`` optional parameters).
         save (bool): Save the new content to the pofile indicated in the
-            parameter ``po_filepath``.
+            parameter ``po_filepath``. If is enabled and ``po_filepath`` is
+            ``None`` a ``ValueError`` will be raised.
         mo_filepath (str): The resulting pofile will be compiled to a mofile
             and saved in the path specified at this parameter.
         plaintext (bool): If you pass ``True`` to this parameter (as default)
@@ -955,8 +1061,8 @@ def markdown_to_pofile(
                 for you. It depends on the use you are going to give to
                 this library activate this mode (``plaintext=False``) or not.
         wrapwidth (int): Wrap width for po file indicated at ``po_filepath``
-            parameter. Only useful when the ``-w`` option was passed
-            to xgettext.
+            parameter. If negative, 0, 'inf' or 'math.inf' the content won't
+            be wrapped.
         mark_not_found_as_obsolete (bool): The strings extracted from markdown
             that will not be found inside the provided pofile will be marked
             as obsolete.
@@ -1021,6 +1127,8 @@ def markdown_to_pofile(
                def msgid_event(self, msgid, *args):
                    if msgid == 'foo':
                        self._disable_next_line = True
+        debug (bool): Add events displaying all parsed elements in the
+            extraction process.
 
     Examples:
         >>> content = 'Some text with `inline code`'
@@ -1032,17 +1140,19 @@ def markdown_to_pofile(
         {'Some text with `inline code`': ''}
         >>> entries = markdown_to_pofile(content, msgstr='Default message')
         >>> {e.msgid: e.msgstr for e in entries}
-        {'Some text with inline code': 'Default message'}
+        {'Some text with `inline code`': 'Default message'}
 
     Returns:
         :class:`polib.POFile` Pofile instance with new msgids included.
+
+    Raises
+        ValueError: when ``po_filepath`` is ``None`` and ``save`` is ``True``.
     """
     return Md2Po(
-        glob_or_content,
+        files_or_content,
         ignore=ignore,
         msgstr=msgstr,
         plaintext=plaintext,
-        wrapwidth=wrapwidth,
         mark_not_found_as_obsolete=mark_not_found_as_obsolete,
         preserve_not_found=preserve_not_found,
         location=location,
@@ -1053,6 +1163,7 @@ def markdown_to_pofile(
         command_aliases=command_aliases,
         metadata=metadata,
         events=events,
+        debug=debug,
         **kwargs,
     ).extract(
         po_filepath=po_filepath,
@@ -1060,4 +1171,5 @@ def markdown_to_pofile(
         mo_filepath=mo_filepath,
         po_encoding=po_encoding,
         md_encoding=md_encoding,
+        wrapwidth=wrapwidth,
     )
