@@ -3,6 +3,7 @@
 import html
 import re
 import warnings
+from collections import OrderedDict
 from html.parser import HTMLParser
 
 import md4c
@@ -12,8 +13,8 @@ from mdpo.command import (
     normalize_mdpo_command_aliases,
     parse_mdpo_html_command,
 )
-from mdpo.html import get_html_attrs_tuple_attr, html_attrs_tuple_to_string
-from mdpo.io import to_file_content_if_is_file
+from mdpo.io import save_file_checking_file_changed, to_file_content_if_is_file
+from mdpo.md import solve_link_reference_targets
 from mdpo.po import (
     paths_or_globs_to_unique_pofiles,
     pofiles_to_unique_translations_dicts,
@@ -27,6 +28,24 @@ PROCESS_REPLACER_TAGS = [
 ALIGNMENT_CHARS = ['\n', ' ', '\t', '\r']
 
 
+def _html_attrs_to_str(attrs):
+    """Converts a dictionary of HTML attributes to its HTML representation.
+
+    Args:
+        attrs (dict): Attributes with values
+
+    Returns:
+        str: HTML attributes string ready to be used inside a HTML tag.
+    """
+    response = ''
+    for name, value in attrs.items():
+        response += name
+        if value is not None:
+            response += f'="{value}"'
+        response += ' '
+    return response.rstrip(' ')
+
+
 class MdPo2HTML(HTMLParser):
     def __init__(
         self,
@@ -41,6 +60,7 @@ class MdPo2HTML(HTMLParser):
         ignore_grouper_tags=['div', 'hr'],
         po_encoding=None,
         command_aliases={},
+        _check_saved_files_changed=None,
     ):
         self.pofiles = paths_or_globs_to_unique_pofiles(
             pofiles,
@@ -67,6 +87,16 @@ class MdPo2HTML(HTMLParser):
         # lazy translators mode
         self.merge_adjacent_markups = merge_adjacent_markups
 
+        # pre-commit mode
+        self._saved_files_changed = (
+            False if _check_saved_files_changed is not None else None
+        )
+
+        # references of msgids and their msgstrs changing referenced link
+        # targets by their real target, needed to support link references
+        # (this variable is redefined when the first link tag `a` is found)
+        self.real_link_reference_targets = None
+
         # code markup
         self.code_tags = code_tags
 
@@ -117,7 +147,7 @@ class MdPo2HTML(HTMLParser):
             self.output = '\n'.join(split_output)[:-1]
 
     def _process_replacer(self):
-        # print("REPLACER:", self.replacer)
+        # print('REPLACER:', self.replacer)
 
         template_tags = []
         raw_html_template, _current_replacement = ('', '')
@@ -132,62 +162,42 @@ class MdPo2HTML(HTMLParser):
             if handle == 'start':
                 template_tags.append(handled)
 
+                attrs_repr = (
+                    f' {_html_attrs_to_str(attrs)}' if attrs else ''
+                )
+
                 if handled in self.code_tags:
                     _current_replacement += '`'
-                    raw_html_template += '<{}{}>'.format(
-                        handled,
-                        (
-                            ' ' + html_attrs_tuple_to_string(attrs)
-                            if attrs else ''
-                        ),
-                    )
+                    raw_html_template += f'<{handled}{attrs_repr}>'
                 elif handled in self.bold_tags:
                     _current_replacement += '**'
-                    raw_html_template += '<{}{}>'.format(
-                        handled,
-                        (
-                            ' ' + html_attrs_tuple_to_string(attrs)
-                            if attrs else ''
-                        ),
-                    )
+                    raw_html_template += f'<{handled}{attrs_repr}>'
                 elif handled in self.italic_tags:
                     _current_replacement += '*'
-                    raw_html_template += '<{}{}>'.format(
-                        handled,
-                        (
-                            ' ' + html_attrs_tuple_to_string(attrs)
-                            if attrs else ''
-                        ),
-                    )
+                    raw_html_template += f'<{handled}{attrs_repr}>'
                 elif handled in self.link_tags:
-                    title = get_html_attrs_tuple_attr(attrs, 'title')
-                    _current_link_target += '(%s' % get_html_attrs_tuple_attr(
-                        attrs, 'href',
-                    )
+                    title, href = attrs.get('title'), attrs['href']
+                    _current_link_target += f'({href}'
                     if title:
-                        _current_link_target += ' "%s"' % title
+                        _current_link_target += f' "{title}"'
                     _current_link_target += ')'
 
-                    raw_html_template += '<%s' % handled
+                    raw_html_template += f'<{handled}'
 
                     # attrs_except_href_title = []
-                    for attr, value in attrs:
+                    for attr, value in attrs.items():
                         if attr in ['title', 'href']:
-                            raw_html_template += ' %s="{}"' % attr
+                            raw_html_template += f' {attr}="{{}}"'
                         # else:
                         #     These attributes are not included in output
                         #    attrs_except_href_title.append((attr, value))
                     # if attrs_except_href_title:
                     #     raw_html_template += ' '
-                    # raw_html_template += html_attrs_tuple_to_string(
+                    # raw_html_template += _html_attrs_to_str(
                     #     attrs_except_href_title) + '>'
                     raw_html_template += '>'
                 else:
-                    raw_html_template += '<{}{}>'.format(
-                        handled,
-                        ' ' + html_attrs_tuple_to_string(attrs)
-                        if attrs else '',
-                    )
+                    raw_html_template += f'<{handled}{attrs_repr}>'
                 _last_start_tag = handled
                 if _last_start_tag == 'code':
                     _inside_code = True
@@ -202,14 +212,14 @@ class MdPo2HTML(HTMLParser):
                 else:
                     raw_html_template += '{}'
                     if _current_link_target:
-                        _current_replacement += '[{}]{}'.format(
-                            handled, _current_link_target,
+                        _current_replacement += (
+                            f'[{handled}]{_current_link_target}'
                         )
                         _current_link_target = ''
                     else:
                         _current_replacement += handled
             elif handle == 'end':
-                raw_html_template += '</%s>' % handled
+                raw_html_template += f'</{handled}>'
                 if handled in self.code_tags:
                     _current_replacement += '`'
                 elif handled in self.bold_tags:
@@ -220,27 +230,22 @@ class MdPo2HTML(HTMLParser):
                 if _last_end_tag == 'code':
                     _inside_code = False
             elif handle == 'comment':
-                raw_html_template += '<!--%s-->' % handled
+                raw_html_template += f'<!--{handled}-->'
             elif handle == 'startend':
                 if handled in self.image_tags:
-                    _current_replacement += '![{}]({}'.format(
-                        get_html_attrs_tuple_attr(attrs, 'alt'),
-                        get_html_attrs_tuple_attr(attrs, 'src'),
-                    )
-                    title = get_html_attrs_tuple_attr(attrs, 'title')
+                    alt, src = attrs.get('alt', ''), attrs['src']
+                    _current_replacement += f'![{alt}]({src}'
+                    title = attrs.get('title')
                     if title:
-                        _current_replacement += ' "%s"' % title
+                        _current_replacement += f' "{title}"'
                     _current_replacement += ')'
 
                     raw_html_template += '{}'
                 else:
-                    raw_html_template += '<{}{}/>'.format(
-                        handled,
-                        (
-                            (' %s' % html_attrs_tuple_to_string(attrs))
-                            if attrs else ''
-                        ),
+                    attrs_repr = (
+                        f' {_html_attrs_to_str(attrs)}' if attrs else ''
                     )
+                    raw_html_template += f'<{handled}{attrs_repr}/>'
 
         _current_replacement = html.unescape(_current_replacement)
 
@@ -272,9 +277,9 @@ class MdPo2HTML(HTMLParser):
                         if tag not in template_tags:
                             template_tags.append(tag)
 
-        # print("RAW TEMPLATE:", raw_html_template)
-        # print("TEMPLATE TAGS:", template_tags)
-        # print('CURRENT MSGID: \'%s\'' % _current_replacement)
+        # print('RAW TEMPLATE:', raw_html_template)
+        # print('TEMPLATE TAGS:', template_tags)
+        # print(f'CURRENT MSGID: \'{_current_replacement}\'')
         # print('MSGSTR:', replacement)
 
         html_before_first_replacement = raw_html_template.split('{')[0]
@@ -282,9 +287,9 @@ class MdPo2HTML(HTMLParser):
         for tags_group in [self.bold_tags, self.italic_tags, self.code_tags]:
             for tag in tags_group:
                 html_before_first_replacement = \
-                    html_before_first_replacement.split('<%s>' % tag)[0]
+                    html_before_first_replacement.split(f'<{tag}>')[0]
                 html_after_last_replacement = \
-                    html_after_last_replacement.split('</%s>' % tag)[-1]
+                    html_after_last_replacement.split(f'</{tag}>')[-1]
             html_before_first_replacement = \
                 html_before_first_replacement.split('<a href="')[0]
 
@@ -310,61 +315,67 @@ class MdPo2HTML(HTMLParser):
         self._enable_next_line = False
         self._current_msgctxt = None
 
-        # print("________________________________________________")
+        # print('________________________________________________')
 
     def handle_starttag(self, tag, attrs):
-        # print("START TAG: %s | POS: %d:%d" % (tag, *self.getpos()))
+        # print('START TAG: %s | POS: %d:%d' % (tag, *self.getpos()))
 
         if tag in self.ignore_grouper_tags:
             self.context.append(tag)
-            self.output += '<{}{}>'.format(
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            attrs_repr = (
+                f' {_html_attrs_to_str(OrderedDict(attrs))}' if attrs else ''
             )
-        elif self.context and self.context[0] in self.ignore_grouper_tags:
-            self.output += '<{}{}>'.format(
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
-            )
-        elif tag == 'ul' and not self.context:
-            self.output += '<{}{}>'.format(
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
-            )
-        elif tag in ['blockquote', 'table', 'thead', 'tbody', 'tr']:
-            self.output += '<{}{}>'.format(
-                tag, ' ' + html_attrs_tuple_to_string(attrs) if attrs else '',
+            self.output += f'<{tag}{attrs_repr}>'
+        elif (
+            (self.context and self.context[0] in self.ignore_grouper_tags) or
+            (tag == 'ul' and not self.context) or
+            (tag in {'blockquote', 'table', 'thead', 'tbody', 'tr'})
+        ):
+            attrs_repr = (
+                f' {_html_attrs_to_str(OrderedDict(attrs))}' if attrs else ''
             )
+            self.output += f'<{tag}{attrs_repr}>'
         else:
-            self.replacer.append(('start', tag, attrs))
+            if tag == 'a' and self.real_link_reference_targets is None:
+                # extend translations to populate link reference msgid-msgstrs
+                # with real targets
+                self.real_link_reference_targets = (
+                    solve_link_reference_targets(self.translations)
+                )
+                self.translations.update(self.real_link_reference_targets)
+
+            self.replacer.append(('start', tag, OrderedDict(attrs)))
             self.context.append(tag)
 
     def handle_endtag(self, tag):
-        # print("END TAG: %s | POS: %d:%d" % (tag, *self.getpos()))
+        # print('END TAG: %s | POS: %d:%d' % (tag, *self.getpos()))
 
         if tag in self.ignore_grouper_tags:
-            self.output += '</%s>' % tag
+            self.output += f'</{tag}>'
             if self.context:
                 self.context.pop()
         elif self.context and self.context[0] in self.ignore_grouper_tags:
-            self.output += '</%s>' % tag
+            self.output += f'</{tag}>'
         elif tag in PROCESS_REPLACER_TAGS:
             self.replacer.append(('end', tag, None))
             self._process_replacer()
         elif tag in ['ul', 'blockquote', 'tr', 'table', 'thead', 'tbody']:
-            self.output += '</%s>' % tag
+            self.output += f'</{tag}>'
         else:
             self.replacer.append(('end', tag, None))
             if self.context:
                 self.context.pop()
 
     def handle_startendtag(self, tag, attrs):
-        # print("STARTEND TAG: %s | POS: %d:%d" % (tag, *self.getpos()))
+        # print('STARTEND TAG: %s | POS: %d:%d' % (tag, *self.getpos()))
 
         if not self.replacer:
             self.output += self.get_starttag_text()
         else:
-            self.replacer.append(('startend', tag, attrs))
+            self.replacer.append(('startend', tag, OrderedDict(attrs)))
 
     def handle_data(self, data):
-        # print("     DATA: '%s'" % (data))
+        # print(f'     DATA: \'{data}\'')
 
         if data:
             if not self.replacer or (
@@ -373,14 +384,13 @@ class MdPo2HTML(HTMLParser):
             ) or \
                     not self.context:
                 self.output += data
-                return
             else:
                 if self.context:
                     data = data.replace('\n', ' ')
                 self.replacer.append(('data', data, None))
 
     def handle_comment(self, data):
-        # print("     COMMENT: '%s'" % (data))
+        # print(f'     COMMENT: \'{data}\'')
 
         if self.replacer:
             self.replacer.append(('comment', data, None))
@@ -431,8 +441,15 @@ class MdPo2HTML(HTMLParser):
         self.feed(content)
 
         if save:
-            with open(save, 'w', encoding=html_encoding) as f:
-                f.write(self.output)
+            if self._saved_files_changed is False:
+                self._saved_files_changed = save_file_checking_file_changed(
+                    save,
+                    self.output,
+                    encoding=html_encoding,
+                )
+            else:
+                with open(save, 'w', encoding=html_encoding) as f:
+                    f.write(self.output)
 
         self.reset()
 
