@@ -7,17 +7,16 @@ import itertools
 import sys
 
 from mdpo.cli import (
+    add_command_alias_argument,
     add_common_cli_first_arguments,
-    add_common_cli_latest_arguments,
+    add_pre_commit_option,
     parse_command_aliases_cli_arguments,
 )
-from mdpo.mdpo2html import markdown_pofile_to_html
+from mdpo.context import environ
+from mdpo.mdpo2html import MdPo2HTML
 
 
-DESCRIPTION = (
-    'HTML-produced-from-Markdown files translator using PO files'
-    ' as reference.'
-)
+DESCRIPTION = 'HTML-produced-from-Markdown file translator using PO files.'
 
 
 def build_parser():
@@ -26,11 +25,12 @@ def build_parser():
     parser.add_argument(
         'filepath_or_content', metavar='FILEPATH_OR_CONTENT',
         nargs='*',
-        help='HTML filepath or content to translate. If not provided, will be'
+        help='HTML file path or content to translate. If not provided, will be'
              ' read from STDIN.',
     )
     parser.add_argument(
-        '-p', '--pofiles', metavar='POFILES', action='append', nargs='*',
+        '-p', '--po-files', '--pofiles', metavar='POFILES', action='append',
+        nargs='*', dest='pofiles',
         help='Glob matching a set of PO files from where to extract references'
              ' to make the replacements translating strings. This argument'
              ' can be passed multiple times.',
@@ -58,7 +58,8 @@ def build_parser():
              ' charset=<ENCODING>\\n".',
         metavar='<ENCODING>',
     )
-    add_common_cli_latest_arguments(parser)
+    add_command_alias_argument(parser)
+    add_pre_commit_option(parser)
     return parser
 
 
@@ -66,13 +67,18 @@ def parse_options(args):
     parser = build_parser()
     if '-h' in args or '--help' in args:
         parser.print_help()
-        sys.exit(0)
+        sys.exit(1)
     opts = parser.parse_args(args)
 
+    filepath_or_content = ''
     if not sys.stdin.isatty():
-        opts.filepath_or_content = sys.stdin.read().strip('\n')
-    elif isinstance(opts.filepath_or_content, list):
-        opts.filepath_or_content = opts.filepath_or_content[0]
+        filepath_or_content += sys.stdin.read().strip('\n')
+    if (
+        isinstance(opts.filepath_or_content, list)
+        and opts.filepath_or_content
+    ):
+        filepath_or_content += opts.filepath_or_content[0]
+    opts.filepath_or_content = filepath_or_content
 
     opts.command_aliases = parse_command_aliases_cli_arguments(
         opts.command_aliases,
@@ -84,18 +90,27 @@ def parse_options(args):
 
 
 def run(args=[]):
-    opts = parse_options(args)
+    with environ(_MDPO_RUNNING='true'):
+        opts = parse_options(args)
 
-    output = markdown_pofile_to_html(
-        opts.filepath_or_content, opts.pofiles,
-        ignore=opts.ignore, save=opts.save,
-        po_encoding=opts.po_encoding,
-        html_encoding=opts.html_encoding,
-        command_aliases=opts.command_aliases,
-    )
+        mdpo2html = MdPo2HTML(
+            opts.pofiles,
+            ignore=opts.ignore,
+            po_encoding=opts.po_encoding,
+            command_aliases=opts.command_aliases,
+            _check_saved_files_changed=opts.check_saved_files_changed,
+        )
+        output = mdpo2html.translate(
+            opts.filepath_or_content,
+            save=opts.save,
+            html_encoding=opts.html_encoding,
+        )
 
-    if not opts.quiet and not opts.save:
-        sys.stdout.write(output + '\n')
+        if not opts.quiet and not opts.save:
+            sys.stdout.write(f'{output}\n')
+
+        if opts.check_saved_files_changed and mdpo2html._saved_files_changed:
+            return (output, 1)
 
     return (output, 0)
 
